#! /bin/sh
# check script for Zutils - Utilities dealing with compressed files
# Copyright (C) 2009-2019 Antonio Diaz Diaz.
#
# This script is free software: you have unlimited permission
# to copy, distribute and modify it.

LC_ALL=C
export LC_ALL
objdir=`pwd`
testdir=`cd "$1" ; pwd`
ZCAT="${objdir}"/zcat
ZCMP="${objdir}"/zcmp
ZDIFF="${objdir}"/zdiff
ZGREP="${objdir}"/zgrep
ZEGREP="${objdir}"/zegrep
ZFGREP="${objdir}"/zfgrep
ZTEST="${objdir}"/ztest
ZUPDATE="${objdir}"/zupdate
compressors="bzip2 gzip lzip"
extensions="bz2 gz lz"
compressor_needed() { echo "${compressors} are needed to run tests" ; exit 1 ; }
framework_failure() { echo "failure in testing framework" ; exit 1 ; }

if [ ! -f "${ZUPDATE}" ] || [ ! -x "${ZUPDATE}" ] ; then
	echo "${ZUPDATE}: cannot execute"
	exit 1
fi

[ -e "${ZUPDATE}" ] 2> /dev/null ||
	{
	echo "$0: a POSIX shell is required to run the tests"
	echo "Try bash -c \"$0 $1 $2\""
	exit 1
	}

if [ -d tmp ] ; then rm -rf tmp ; fi
mkdir tmp
cd "${objdir}"/tmp || framework_failure

for i in ${compressors}; do
	cat "${testdir}"/test.txt > in || framework_failure
	$i in || compressor_needed
	printf "Hello World!\n" > hello || framework_failure
	$i hello || compressor_needed
done

cat "${testdir}"/test.txt > in || framework_failure
cat "${testdir}"/test.txt.tar > in.tar || framework_failure
printf "01234567890" > pin.tar || framework_failure
cat in.tar in.tar in.tar in.tar >> pin.tar || framework_failure
cat in > -in- || framework_failure
cat in.lz > -in-.lz || framework_failure
cat in.lz > lz_only.lz || framework_failure
cat in in in in in in > in6 || framework_failure
bad0_lz="${testdir}"/zero_bad_crc.lz
bad0_gz="${testdir}"/zero_bad_crc.gz
bad1_lz="${testdir}"/test_bad_crc.lz
fail=0
test_failed() { fail=1 ; printf " $1" ; [ -z "$2" ] || printf "($2)" ; }

printf "testing    zcat-%s..." "$2"

for i in ${extensions}; do
	"${ZCAT}" -N in.$i > copy || test_failed $LINENO $i
	cmp in copy || test_failed $LINENO $i
	"${ZCAT}" -N --format=un in.$i > copy || test_failed $LINENO $i
	cmp in copy || test_failed $LINENO $i
	"${ZCAT}" -N --force-format=$i in.$i > copy || test_failed $LINENO $i
	cmp in copy || test_failed $LINENO $i
	"${ZCAT}" -N in.$i | dd bs=1000 count=1 > copy 2> /dev/null ||
		test_failed $LINENO $i
	dd if=in bs=1000 count=1 2> /dev/null | cmp - copy ||
		test_failed $LINENO $i
done

"${ZCAT}" -N -v -s "${testdir}"/zcat_vs.dat > /dev/null || test_failed $LINENO
"${ZCAT}" -N < in > copy || test_failed $LINENO
cmp in copy || test_failed $LINENO
"${ZCAT}" -N < in.gz > copy || test_failed $LINENO
cmp in copy || test_failed $LINENO
"${ZCAT}" -N < in.bz2 > copy || test_failed $LINENO
cmp in copy || test_failed $LINENO
"${ZCAT}" -N < in.lz > copy || test_failed $LINENO
cmp in copy || test_failed $LINENO
"${ZCAT}" -N -O lz - - < in.lz > copy || test_failed $LINENO
cmp in copy || test_failed $LINENO
"${ZCAT}" -N --lz='lzip -q' < in.lz > copy || test_failed $LINENO
cmp in copy || test_failed $LINENO
"${ZCAT}" -N in > copy || test_failed $LINENO
cmp in copy || test_failed $LINENO
"${ZCAT}" -N lz_only > copy || test_failed $LINENO
cmp in copy || test_failed $LINENO
"${ZCAT}" -N in in.gz in.bz2 in.lz -- -in- -in-.lz > copy || test_failed $LINENO
cmp in6 copy || test_failed $LINENO
"${ZCAT}" -Nq in in.gz in.bz2 in.lz "${bad0_lz}" -- -in- -in-.lz > copy
[ $? = 1 ] || test_failed $LINENO
cmp in6 copy || test_failed $LINENO
"${ZCAT}" -Nq "${bad1_lz}" -- -in-.lz in in.gz in.bz2 in.lz > copy
[ $? = 1 ] || test_failed $LINENO
cmp in6 copy || test_failed $LINENO
"${ZCAT}" -N . || test_failed $LINENO
"${ZCAT}" -N -r . > /dev/null || test_failed $LINENO
"${ZCAT}" -N -r > /dev/null || test_failed $LINENO
"${ZCAT}" -N -R . > /dev/null || test_failed $LINENO
"${ZCAT}" -N -R > /dev/null || test_failed $LINENO

"${ZCAT}" -Nq --format=, in.lz
[ $? = 1 ] || test_failed $LINENO
"${ZCAT}" -Nq --format=,lz in.lz
[ $? = 1 ] || test_failed $LINENO
"${ZCAT}" -Nq --format=gz,,lz in.lz
[ $? = 1 ] || test_failed $LINENO
"${ZCAT}" -Nq --format=lz,, in.lz
[ $? = 1 ] || test_failed $LINENO
"${ZCAT}" -Nq --format=nc in.lz
[ $? = 1 ] || test_failed $LINENO
"${ZCAT}" -Nq --lz='-lzip -q' in.lz
[ $? = 1 ] || test_failed $LINENO
"${ZCAT}" -N --force-format=gz in.bz2 2> /dev/null
[ $? = 1 ] || test_failed $LINENO
"${ZCAT}" -N --force-format=bz2 in.lz 2> /dev/null
[ $? = 1 ] || test_failed $LINENO
"${ZCAT}" -Nq --force-format=lz in.gz
[ $? = 1 ] || test_failed $LINENO
"${ZCAT}" -N --bad-option 2> /dev/null
[ $? = 1 ] || test_failed $LINENO


printf "\ntesting    zcmp-%s..." "$2"

for i in ${extensions}; do
	"${ZCMP}" -N in.$i || test_failed $LINENO $i
	"${ZCMP}" -N in in.$i || test_failed $LINENO $i
	"${ZCMP}" -N -i 1kB:1000 -n 500 in6 in.$i || test_failed $LINENO $i
	"${ZCMP}" -N in in.$i --force-format=,$i || test_failed $LINENO $i
	"${ZCMP}" -N in.$i in || test_failed $LINENO $i
	"${ZCMP}" -N -i 1KiB:1024 -n 50 in.$i in6 || test_failed $LINENO $i
	"${ZCMP}" -N in.$i in --force-format=$i || test_failed $LINENO $i
done

"${ZCMP}" -Nq in in6
[ $? = 1 ] || test_failed $LINENO
"${ZCMP}" -N -n 0 in in6 || test_failed $LINENO
"${ZCMP}" -N -n 100B in in6 || test_failed $LINENO
"${ZCMP}" -N -n 1k in in6 || test_failed $LINENO
"${ZCMP}" -N -n 10kB in in6 || test_failed $LINENO
"${ZCMP}" -Nq in.tar pin.tar
[ $? = 1 ] || test_failed $LINENO
"${ZCMP}" -Nq -i 0B:11B in.tar pin.tar
[ $? = 1 ] || test_failed $LINENO
"${ZCMP}" -N -i 0:11 -n 0 in.tar pin.tar || test_failed $LINENO
"${ZCMP}" -N -i 0:11 -n 100 in.tar pin.tar || test_failed $LINENO
"${ZCMP}" -N -i 0:11 -n 1Ki in.tar pin.tar || test_failed $LINENO
"${ZCMP}" -N -i 0:11 -n 10KiB in.tar pin.tar || test_failed $LINENO
"${ZCMP}" -N - || test_failed $LINENO
"${ZCMP}" -N in in || test_failed $LINENO
"${ZCMP}" -N in || test_failed $LINENO
"${ZCMP}" -N in.lz in.gz || test_failed $LINENO
"${ZCMP}" -N --lz='lzip -q' in.lz in.gz || test_failed $LINENO
"${ZCMP}" -N in.gz -- -in-.lz || test_failed $LINENO
"${ZCMP}" -N -- -in-.lz in.gz || test_failed $LINENO
"${ZCMP}" -N in -- -in-.lz || test_failed $LINENO
"${ZCMP}" -N -- -in- in.lz || test_failed $LINENO
"${ZCMP}" -N in.lz -- -in- || test_failed $LINENO
"${ZCMP}" -N -- -in-.lz in || test_failed $LINENO
"${ZCMP}" -N -- -in- in || test_failed $LINENO
"${ZCMP}" -N in -- -in- || test_failed $LINENO
"${ZCMP}" -N lz_only.lz < in || test_failed $LINENO
"${ZCMP}" -N in.lz - < in || test_failed $LINENO
"${ZCMP}" -N - in.lz < in || test_failed $LINENO
"${ZCMP}" -N in - < in.lz || test_failed $LINENO
"${ZCMP}" -N - in < in.lz || test_failed $LINENO
"${ZCMP}" -N -q --force-format=lz in.lz
[ $? = 2 ] || test_failed $LINENO
"${ZCMP}" -Nq --force-format=lz in.gz in.lz
r=$?
{ [ $r = 1 ] || [ $r = 2 ] ; } || test_failed $LINENO
"${ZCMP}" -Nq -i 100BB in in
[ $? = 2 ] || test_failed $LINENO
"${ZCMP}" -Nq -i 100BB:100 in in
[ $? = 2 ] || test_failed $LINENO
"${ZCMP}" -Nq -i 100: in in
[ $? = 2 ] || test_failed $LINENO
"${ZCMP}" -Nq -n -1 in in
[ $? = 2 ] || test_failed $LINENO
"${ZCMP}" -Nq -n 100BB in in
[ $? = 2 ] || test_failed $LINENO
"${ZCMP}" -N --bad-option in in 2> /dev/null
[ $? = 2 ] || test_failed $LINENO


printf "\ntesting   zdiff-%s..." "$2"

for i in ${extensions}; do
	"${ZDIFF}" -N in.$i > /dev/null || test_failed $LINENO $i
	"${ZDIFF}" -N in in.$i > /dev/null || test_failed $LINENO $i
	"${ZDIFF}" -N --force-format=,$i in in.$i > /dev/null ||
		test_failed $LINENO $i
	"${ZDIFF}" -N in.$i in > /dev/null || test_failed $LINENO $i
	"${ZDIFF}" -N --force-format=$i, in.$i in > /dev/null ||
		test_failed $LINENO $i
done

"${ZDIFF}" -N in in6 > /dev/null && test_failed $LINENO
"${ZDIFF}" -N in.tar pin.tar > /dev/null && test_failed $LINENO
"${ZDIFF}" -N - || test_failed $LINENO
"${ZDIFF}" -N in in || test_failed $LINENO
"${ZDIFF}" -N in || test_failed $LINENO
"${ZDIFF}" -N --format=gz,bz2 in || test_failed $LINENO
"${ZDIFF}" -N --format=gz in || test_failed $LINENO
"${ZDIFF}" -N in.lz in.gz > /dev/null || test_failed $LINENO
"${ZDIFF}" -N --lz='lzip -q' in.lz in.gz > /dev/null || test_failed $LINENO
"${ZDIFF}" -N in.gz -- -in-.lz > /dev/null || test_failed $LINENO
"${ZDIFF}" -N -- -in-.lz in.gz > /dev/null || test_failed $LINENO
"${ZDIFF}" -N in -- -in-.lz > /dev/null || test_failed $LINENO
"${ZDIFF}" -N -- -in- in.lz > /dev/null || test_failed $LINENO
"${ZDIFF}" -N in.lz -- -in- > /dev/null || test_failed $LINENO
"${ZDIFF}" -N -- -in-.lz in > /dev/null || test_failed $LINENO
"${ZDIFF}" -N -- -in- in > /dev/null || test_failed $LINENO
"${ZDIFF}" -N in -- -in- > /dev/null || test_failed $LINENO
"${ZDIFF}" -N lz_only.lz < in > /dev/null || test_failed $LINENO
"${ZDIFF}" -N in.lz - < in > /dev/null || test_failed $LINENO
"${ZDIFF}" -N - in.lz < in > /dev/null || test_failed $LINENO
"${ZDIFF}" -N in - < in.lz > /dev/null || test_failed $LINENO
"${ZDIFF}" -N - in < in.lz > /dev/null || test_failed $LINENO
"${ZDIFF}" -N --bz2='-bzip2' in.bz2 2> /dev/null
[ $? = 2 ] || test_failed $LINENO
"${ZDIFF}" -N -q --force-format=bz2 in.bz2 2> /dev/null
[ $? = 2 ] || test_failed $LINENO
"${ZDIFF}" -N -q --force-format=,lz in.lz in.bz2 > /dev/null 2>&1
r=$?
{ [ $r = 1 ] || [ $r = 2 ] ; } || test_failed $LINENO
"${ZDIFF}" -N --bad-option 2> /dev/null
[ $? = 2 ] || test_failed $LINENO

mkdir tmp2
cat in > tmp2/a || framework_failure
cat in.lz > tmp2/a.lz || framework_failure
"${ZDIFF}" -N --format=bz2 tmp2/a < /dev/null > /dev/null
[ $? = 1 ] || test_failed $LINENO
"${ZDIFF}" -N --format=gz tmp2/a < /dev/null > /dev/null
[ $? = 1 ] || test_failed $LINENO
"${ZDIFF}" -N --format=lz tmp2/a.lz < /dev/null > /dev/null
[ $? = 1 ] || test_failed $LINENO
"${ZDIFF}" -N --format=lz tmp2/a < /dev/null || test_failed $LINENO
"${ZDIFF}" -N --format=un tmp2/a.lz < /dev/null || test_failed $LINENO
rm -r tmp2 || framework_failure


printf "\ntesting   zgrep-%s..." "$2"

for i in ${extensions}; do
	"${ZGREP}" -N "GNU" in.$i > /dev/null || test_failed $LINENO $i
	"${ZGREP}" -N "GNU" in.$i hello.$i > /dev/null || test_failed $LINENO $i
	"${ZGREP}" -N "GNU" hello.$i in.$i > /dev/null || test_failed $LINENO $i
	"${ZGREP}" -N -q "GNU" in.$i hello.$i || test_failed $LINENO $i
	"${ZGREP}" -N -q "GNU" hello.$i in.$i || test_failed $LINENO $i
	"${ZGREP}" -N "GNU" < in.$i > /dev/null || test_failed $LINENO $i
	"${ZGREP}" -N -l "GNU" in.$i > /dev/null || test_failed $LINENO $i
	"${ZGREP}" -N -L "GNU" in.$i || test_failed $LINENO $i
	"${ZGREP}" -N --force-format=$i "GNU" in.$i > /dev/null ||
		test_failed $LINENO $i
	"${ZGREP}" -N -v "nx_pattern" in.$i > /dev/null ||
		test_failed $LINENO $i
	"${ZGREP}" -N "nx_pattern" in.$i && test_failed $LINENO $i
	"${ZGREP}" -N -l "nx_pattern" in.$i && test_failed $LINENO $i
	"${ZGREP}" -N -L "nx_pattern" in.$i > /dev/null &&
		test_failed $LINENO $i
	"${ZGREP}" -N --force-format=$i "GNU" in 2> /dev/null
	[ $? = 2 ] || test_failed $LINENO $i
done

"${ZGREP}" -N pin.tar -e "GNU" > /dev/null || test_failed $LINENO
"${ZGREP}" -N "GNU" < pin.tar > /dev/null || test_failed $LINENO
"${ZGREP}" -N -r "GNU" . > /dev/null || test_failed $LINENO
"${ZGREP}" -N -r "GNU" > /dev/null || test_failed $LINENO
"${ZGREP}" -N -R "GNU" . > /dev/null || test_failed $LINENO
"${ZGREP}" -N -R "GNU" > /dev/null || test_failed $LINENO
"${ZGREP}" -N "nx_pattern" -r . in > /dev/null && test_failed $LINENO
"${ZGREP}" -N -e "GNU" in > /dev/null || test_failed $LINENO
"${ZGREP}" -N "GNU" < in > /dev/null || test_failed $LINENO
"${ZGREP}" -N -O lz "nx_pattern" - - < in.lz > /dev/null && test_failed $LINENO
"${ZGREP}" -N -e "-free" --lz='lzip -q' < in.lz > /dev/null ||
	test_failed $LINENO
"${ZGREP}" -N -- "-free" -in- > /dev/null || test_failed $LINENO
"${ZGREP}" -N -q -- "-free" nx_file -in-.lz || test_failed $LINENO
"${ZGREP}" -N "GNU" in in.gz in.bz2 in.lz -- -in- > /dev/null ||
	test_failed $LINENO
"${ZGREP}" -N -l "GNU" in in.gz in.bz2 in.lz -- -in- > /dev/null ||
	test_failed $LINENO
"${ZGREP}" -N -L "GNU" in in.gz in.bz2 in.lz -- -in- || test_failed $LINENO
"${ZGREP}" -N -l "nx_pattern" in in.gz in.bz2 in.lz -- -in- &&
	test_failed $LINENO
"${ZGREP}" -N -L "nx_pattern" in in.gz in.bz2 in.lz -- -in- > /dev/null &&
	test_failed $LINENO
"${ZGREP}" -Nq -l "01234567890" in "${bad1_lz}" in.lz && test_failed $LINENO
"${ZGREP}" -Nq -l "01234567890" in "${bad1_lz}" in.lz pin.tar > /dev/null ||
	test_failed $LINENO

"${ZGREP}" -N "GNU" .
[ $? = 1 ] || test_failed $LINENO
"${ZGREP}" -N --bad-option 2> /dev/null
[ $? = 2 ] || test_failed $LINENO
"${ZGREP}" -N "GNU" -s nx_file
[ $? = 2 ] || test_failed $LINENO
"${ZGREP}" -N -q
[ $? = 2 ] || test_failed $LINENO

"${ZEGREP}" -N "GNU" in > /dev/null || test_failed $LINENO
"${ZFGREP}" -N "GNU" in > /dev/null || test_failed $LINENO


printf "\ntesting   ztest-%s..." "$2"

for i in ${extensions}; do
	"${ZTEST}" -N --force-format=$i < in.$i || test_failed $LINENO $i
	"${ZTEST}" -N --force-format=$i < in 2> /dev/null
	[ $? = 2 ] || test_failed $LINENO $i
	"${ZTEST}" -N --force-format=$i in 2> /dev/null
	[ $? = 2 ] || test_failed $LINENO $i
done

"${ZTEST}" -N in in.gz in.bz2 in.lz -- -in- || test_failed $LINENO
"${ZTEST}" -N < in.gz || test_failed $LINENO
"${ZTEST}" -N < in.bz2 || test_failed $LINENO
"${ZTEST}" -N < in.lz || test_failed $LINENO
"${ZTEST}" -N - < in.lz || test_failed $LINENO
"${ZTEST}" -N - in.gz - < in.lz || test_failed $LINENO
"${ZTEST}" -N --lz='lzip -q' < in.lz || test_failed $LINENO
"${ZTEST}" -N -r . || test_failed $LINENO
"${ZTEST}" -N -r || test_failed $LINENO
"${ZTEST}" -N -R . || test_failed $LINENO
"${ZTEST}" -N -R || test_failed $LINENO

"${ZTEST}" -Nq in.gz "${bad0_lz}" in.bz2 "${bad1_lz}" in.lz
[ $? = 2 ] || test_failed $LINENO
lines=`"${ZTEST}" -N in.gz "${bad0_lz}" in.bz2 "${bad1_lz}" in.lz 2>&1 | wc -l`
[ "${lines}" -eq 2 ] || test_failed $LINENO
lines=`"${ZTEST}" -Nv in.gz "${bad0_lz}" in.bz2 "${bad1_lz}" in.lz 2>&1 | wc -l`
[ "${lines}" -eq 5 ] || test_failed $LINENO
"${ZTEST}" -Nq < in
[ $? = 2 ] || test_failed $LINENO
dd if=in.lz bs=1000 count=1 2> /dev/null | "${ZTEST}" -N -q
[ $? = 2 ] || test_failed $LINENO
"${ZTEST}" -Nq --force-format=lz in.bz2
[ $? = 2 ] || test_failed $LINENO
"${ZTEST}" -N --lz='lzip --bad-option' in.lz 2> /dev/null
[ $? = 1 ] || test_failed $LINENO
"${ZTEST}" -N --bad-option 2> /dev/null
[ $? = 1 ] || test_failed $LINENO


printf "\ntesting zupdate-%s..." "$2"

cat in.bz2 > a.bz2 || framework_failure
cat in.gz > a.gz || framework_failure
"${ZUPDATE}" -Nq --bz2=bad_command a.bz2
[ $? = 1 ] || test_failed $LINENO
"${ZUPDATE}" -N --bz2='bzip2 --bad-option' a.bz2 > /dev/null 2>&1
[ $? = 1 ] || test_failed $LINENO
"${ZUPDATE}" -Nq --gz=bad_command a.gz
[ $? = 1 ] || test_failed $LINENO
"${ZUPDATE}" -N --gz='gzip --bad-option' a.gz 2> /dev/null
[ $? = 1 ] || test_failed $LINENO
"${ZUPDATE}" -Nq --lz=bad_command a.gz
[ $? = 1 ] || test_failed $LINENO
"${ZUPDATE}" -N --lz='lzip --bad-option' a.gz 2> /dev/null
[ $? = 1 ] || test_failed $LINENO
"${ZUPDATE}" -N --bad-option 2> /dev/null
[ $? = 1 ] || test_failed $LINENO

cat in.lz in.lz > a.lz || framework_failure
"${ZUPDATE}" -Nq -f a.bz2 a.gz
[ $? = 1 ] || test_failed $LINENO
[ -e a.bz2 ] || test_failed $LINENO
[ -e a.gz ] || test_failed $LINENO
[ -e a.lz ] || test_failed $LINENO
rm -f a.lz || framework_failure

"${ZUPDATE}" -N a.bz2 || test_failed $LINENO
[ ! -e a.bz2 ] || test_failed $LINENO
[ -e a.gz ] || test_failed $LINENO
[ -e a.lz ] || test_failed $LINENO
rm -f a.lz || framework_failure
"${ZUPDATE}" -N a.gz || test_failed $LINENO
[ ! -e a.bz2 ] || test_failed $LINENO
[ ! -e a.gz ] || test_failed $LINENO
[ -e a.lz ] || test_failed $LINENO
rm -f a.lz || framework_failure

cat in.bz2 > a.bz2 || framework_failure
cat in.gz > a.gz || framework_failure
"${ZUPDATE}" -Nq a.bz2 a.gz
[ $? = 1 ] || test_failed $LINENO
[ ! -e a.bz2 ] || test_failed $LINENO
[ -e a.gz ] || test_failed $LINENO
[ -e a.lz ] || test_failed $LINENO
rm -f a.lz || framework_failure

cat in.bz2 > a.bz2 || framework_failure
cat in.gz > a.gz || framework_failure
"${ZUPDATE}" -N -f -k a.bz2 a.gz || test_failed $LINENO
[ -e a.bz2 ] || test_failed $LINENO
[ -e a.gz ] || test_failed $LINENO
[ -e a.lz ] || test_failed $LINENO
rm -f a.lz || framework_failure

cat in.bz2 > a.bz2 || framework_failure
cat in.gz > a.gz || framework_failure
"${ZUPDATE}" -N -f a.bz2 a.gz || test_failed $LINENO
[ ! -e a.bz2 ] || test_failed $LINENO
[ ! -e a.gz ] || test_failed $LINENO
[ ! -e a ] || test_failed $LINENO
[ -e a.lz ] || test_failed $LINENO
rm -f a.lz || framework_failure

cat in.bz2 > a.bz2 || framework_failure
cat "${bad0_gz}" > b.gz || framework_failure
cat in.gz > c.gz || framework_failure
"${ZUPDATE}" -N -f a.bz2 b.gz c.gz 2> /dev/null && test_failed $LINENO
[ ! -e a.bz2 ] || test_failed $LINENO
[ -e b.gz ] || test_failed $LINENO
[ -e c.gz ] || test_failed $LINENO
[ ! -e a ] || test_failed $LINENO
[ ! -e b ] || test_failed $LINENO
[ ! -e c ] || test_failed $LINENO
[ -e a.lz ] || test_failed $LINENO
rm -f a.lz b.gz c.gz || framework_failure

cat in.bz2 > a.bz2 || framework_failure
"${ZUPDATE}" -N -1 -q a.bz2 || test_failed $LINENO
[ ! -e a.bz2 ] || test_failed $LINENO
[ -e a.lz ] || test_failed $LINENO
rm -f a.lz || framework_failure

mkdir tmp2
mkdir tmp2/tmp3
cat in.bz2 > tmp2/tmp3/a.bz2 || framework_failure
cat in.gz > tmp2/tmp3/a.gz || framework_failure
"${ZUPDATE}" -N -r --format=gz tmp2 || test_failed $LINENO
[ -e tmp2/tmp3/a.bz2 ] || test_failed $LINENO
[ ! -e tmp2/tmp3/a.gz ] || test_failed $LINENO
[ -e tmp2/tmp3/a.lz ] || test_failed $LINENO
rm -f tmp2/tmp3/a.lz || framework_failure
"${ZUPDATE}" -N -r --format=bz2 tmp2 || test_failed $LINENO
[ ! -e tmp2/tmp3/a.bz2 ] || test_failed $LINENO
[ ! -e tmp2/tmp3/a.gz ] || test_failed $LINENO
[ -e tmp2/tmp3/a.lz ] || test_failed $LINENO
rm -f tmp2/tmp3/a.lz || framework_failure
cat in.bz2 > tmp2/tmp3/a.bz2 || framework_failure
cat in.gz > tmp2/tmp3/a.gz || framework_failure
cd tmp2 || framework_failure
"${ZUPDATE}" -N -r -k -f . || test_failed $LINENO
[ -e tmp3/a.bz2 ] || test_failed $LINENO
[ -e tmp3/a.gz ] || test_failed $LINENO
[ -e tmp3/a.lz ] || test_failed $LINENO
rm -f tmp3/a.lz || framework_failure
"${ZUPDATE}" -N -r -k -f || test_failed $LINENO
[ -e tmp3/a.bz2 ] || test_failed $LINENO
[ -e tmp3/a.gz ] || test_failed $LINENO
[ -e tmp3/a.lz ] || test_failed $LINENO
rm -f tmp3/a.lz || framework_failure
"${ZUPDATE}" -N -R -k -f . || test_failed $LINENO
[ -e tmp3/a.bz2 ] || test_failed $LINENO
[ -e tmp3/a.gz ] || test_failed $LINENO
[ -e tmp3/a.lz ] || test_failed $LINENO
rm -f tmp3/a.lz || framework_failure
"${ZUPDATE}" -N -R -k -f || test_failed $LINENO
[ -e tmp3/a.bz2 ] || test_failed $LINENO
[ -e tmp3/a.gz ] || test_failed $LINENO
[ -e tmp3/a.lz ] || test_failed $LINENO
rm -f tmp3/a.lz || framework_failure
"${ZUPDATE}" -N -r -f . || test_failed $LINENO
[ ! -e tmp3/a.bz2 ] || test_failed $LINENO
[ ! -e tmp3/a.gz ] || test_failed $LINENO
[ -e tmp3/a.lz ] || test_failed $LINENO
cd .. || framework_failure
rm -r tmp2 || framework_failure

if ln -s '.' slink 2> /dev/null ; then
	"${ZCAT}" -N -r slink > /dev/null || test_failed $LINENO
	"${ZGREP}" -N -r "GNU" slink > /dev/null || test_failed $LINENO
	"${ZTEST}" -N -r slink || test_failed $LINENO
	"${ZUPDATE}" -N -r -f slink || test_failed $LINENO
else
	printf "\nwarning: skipping link test: 'ln' does not work on your system."
fi
rm -f slink || framework_failure

echo
if [ ${fail} = 0 ] ; then
	echo "tests completed successfully."
	cd "${objdir}" && rm -r tmp
else
	echo "tests failed."
fi
exit ${fail}
